#' ---
#' title: "R packages and version"
#' author: Timo Gnambs
#' date: "`r Sys.time()`"
#' output:
#'    html_document:
#'        toc: true
#'        number_sections: true
#'        toc_float: true
#'        theme: readable
#'        highlight: tango
#'        df_print: paged
#'        code_folding: show
#' ---

# Clear workspace
rm(list = ls())



#
# Read a file line by line and identify loaded libraries
#
# @param   vector  filepath     current file
# @return  vector               identified libraries
#
processFile <- function(filepath) {
    con <- file(filepath, "r")
    libs <- c()
    while (TRUE) {
        line <- readLines(con, n = 1)
        line <- stringr::str_trim(gsub("(#.+)$", "", line)) # remove comments
        if ( length(line) == 0 ) {
            break
        }

        # loaded via "library"
        g <- gregexpr("library\\(([^\\s]+)\\)", line, perl = TRUE,
                      ignore.case = TRUE)
        if (g[[1]] == 1) {
            start <- attr(g[[1]], "capture.start")[1]
            stop <- start + attr(g[[1]], "capture.length")[1] - 1
            libs <- c(libs, substr(line, start = start, stop = stop))
        }

        # directly addressed functions
        g <- gregexpr("([[:alnum:]]+)::", line, perl = TRUE, ignore.case = TRUE)
        if (g[[1]] > 0) {
            start <- attr(g[[1]], "capture.start")[1]
            stop <- start + attr(g[[1]], "capture.length")[1] - 1
            libs <- c(libs, substr(line, start = start, stop = stop))
        }
    }

    close(con)
    libs
}



#' # Gather used libraries

# all R syntax files
files <- list.files(here::here("Code"), recursive = FALSE)

# all loaded libraries
libs <- c()
for (file in files) {

    # only check R files
    if (grepl("..+\\.[R]", file, perl = TRUE,
              ignore.case = TRUE)) {

        # extract loaded libraries
        libs <- c(libs, processFile(here::here("Code", file)))
    }
}
rm(files, file, processFile)
(libs <- sort(unique(libs)))

# detach all packages
basic.packages <- c("package:stats", "package:graphics",
                    "package:grDevices", "package:utils",
                    "package:datasets", "package:methods",
                    "package:base")
package.list <- search()[ifelse(unlist(gregexpr("package:", search())) == 1, TRUE, FALSE)]
package.list <- setdiff(package.list, basic.packages)
if (length(package.list) > 0) {
    for (package in package.list) detach(package, character.only = TRUE)
}
rm(basic.packages, package.list)

# load all packages
for (lib in libs) {
    suppressPackageStartupMessages({
        library(lib, character.only = TRUE, warn.conflicts = FALSE)
    })
}
rm(libs, lib)



#' # R version and packages

sessionInfo()

