#' ---
#' title: Various helper functions
#' author: Timo Gnambs
#' date: "`r Sys.time()`"
#' output:
#'    html_document:
#'        toc: true
#'        number_sections: true
#'        toc_float: true
#'        theme: readable
#'        highlight: tango
#'        code_folding: show
#' ---



# Mode of variable
#
# @source  tidybayes (version 2.0.2)
#
Mode <- function(x, na.rm = TRUE, ties = "rm") {
    if (na.rm) {
        x = x[!is.na(x)]
    } else if (any(is.na(x))) {
        return(NA)
    }
    if (length(x) == 0) return(NA)
    ux <- unique(x)
    fu <- tabulate(match(x, ux))
    if (ties == "rm" & sum(fu == max(fu)) > 1) return(NA)
    ux[which.max(fu)]
}



# Calculate intraclass-correlation from
#   a fitted lmer object
ICC <- function(obj, digits = 3) {
    v <- VarCorr(obj)
    vars <- attr(v, "sc")^2
    for (i in seq_len(length(v))) {
        vars <- c(vars, v[[i]][1])
    }
    icc <- vars[-1] / sum(vars)
    names(icc) <- names(v)
    round(icc, digits)
}



# Calculate an I2-analogous heterogeneity measure
#   from a fitted lmer object
I2 <- function(obj, digits = 3) {
  v <- VarCorr(obj)
  vars <- tibble(day = c("isMo2", "isFr2", "isSa2", "isSu2"),
                 tau2 = NA,
                 v = diag(vcov(obj)[c("isMoMonday", "isFrFriday",
                                      "isSaSaturday", "isSuSunday"),
                                    c("isMoMonday", "isFrFriday",
                                      "isSaSaturday", "isSuSunday")]))
  for (i in seq_len(length(v))) {
    if (colnames(v[[i]]) %in% vars$day) {
      vars$tau2[vars$day == colnames(v[[i]])] <- v[[i]][1]
    }
  }
  i2 <- vars$tau2 / rowSums(vars[, -1])
  names(i2) <- vars$day
  round(i2, digits)
}



# log-likelihood ratio test
anova_ <- function(mod0, mod1) {
    NL0 <- -logLik(mod0)
    NL1 <- -logLik(mod1)
    devdiff <- 2 * (NL0 - NL1)
    dfdiff <- attr(NL1, "df") - attr(NL0, "df")
    if (dfdiff < 0) {
        dfdiff <- dfdiff * -1
        devdiff <- devdiff * -1
    }
    p <- pchisq(devdiff, dfdiff, lower.tail = FALSE)
    c(chi2 = devdiff,
      df = dfdiff,
      p = p)
}
